import _extends from "@babel/runtime/helpers/esm/extends";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import React from 'react';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import hoistNonReactStatic from 'hoist-non-react-statics';
import { connectReadModel as connectReadModelAction, disconnectReadModel as disconnectReadModelAction } from './actions';
import getHash from '../internal/get-hash';
import connectResolveAdvanced from '../internal/connect-resolve-advanced';
import { ResultStatus } from '../types';
import { getEntry } from './read-model-reducer';

var connectReadModel = function connectReadModel(mapStateToOptions) {
  return function (Component) {
    var ReadModelContainer = /*#__PURE__*/function (_React$PureComponent) {
      _inheritsLoose(ReadModelContainer, _React$PureComponent);

      function ReadModelContainer() {
        return _React$PureComponent.apply(this, arguments) || this;
      }

      var _proto = ReadModelContainer.prototype;

      _proto.componentDidMount = function componentDidMount() {
        var _this$props$connector = this.props.connectorOptions,
            readModelName = _this$props$connector.readModelName,
            resolverName = _this$props$connector.resolverName,
            resolverArgs = _this$props$connector.resolverArgs;
        this.props.connectReadModel({
          name: readModelName,
          resolver: resolverName,
          args: resolverArgs
        });
      };

      _proto.componentWillUnmount = function componentWillUnmount() {
        var _this$props$connector2 = this.props.connectorOptions,
            readModelName = _this$props$connector2.readModelName,
            resolverName = _this$props$connector2.resolverName,
            resolverArgs = _this$props$connector2.resolverArgs;
        this.props.disconnectReadModel({
          name: readModelName,
          resolver: resolverName,
          args: resolverArgs
        });
      };

      _proto.componentDidUpdate = function componentDidUpdate(prevProps) {
        var connectorOptions = this.props.connectorOptions;
        var prevConnectorOptions = prevProps.connectorOptions;

        if (connectorOptions && prevConnectorOptions && (prevConnectorOptions.readModelName !== connectorOptions.readModelName || prevConnectorOptions.resolverName !== connectorOptions.resolverName || prevConnectorOptions.resolverArgs !== connectorOptions.resolverArgs) && (prevConnectorOptions.readModelName !== connectorOptions.readModelName || prevConnectorOptions.resolverName !== connectorOptions.resolverName || getHash(prevConnectorOptions.resolverArgs) !== getHash(connectorOptions.resolverArgs))) {
          this.props.disconnectReadModel({
            name: prevConnectorOptions.readModelName,
            resolver: prevConnectorOptions.resolverName,
            args: prevConnectorOptions.resolverArgs
          });
          this.props.connectReadModel({
            name: connectorOptions.readModelName,
            resolver: connectorOptions.resolverName,
            args: connectorOptions.resolverArgs
          });
        }
      };

      _proto.render = function render() {
        var _this$props = this.props,
            ownProps = _this$props.ownProps,
            isLoading = _this$props.isLoading,
            data = _this$props.data;
        return /*#__PURE__*/React.createElement(Component, _extends({}, ownProps, {
          isLoading: isLoading,
          data: data
        }));
      };

      return ReadModelContainer;
    }(React.PureComponent);

    var mapStateToConnectorProps = function mapStateToConnectorProps(state, ownProps) {
      var connectorOptions = mapStateToOptions(state, ownProps);
      var entry = getEntry(state.readModels, {
        query: {
          name: connectorOptions.readModelName,
          resolver: connectorOptions.resolverName,
          args: connectorOptions.resolverArgs
        }
      });
      var data = entry && entry.status === ResultStatus.Ready ? entry.data : null;
      var error = entry && entry.status === ResultStatus.Failed ? entry.error : null;
      return {
        ownProps: ownProps,
        connectorOptions: connectorOptions,
        isLoading: entry && entry.status === ResultStatus.Requested,
        isFailure: entry && entry.status === ResultStatus.Failed,
        data: data,
        error: error
      };
    };

    var mapDispatchToConnectorProps = function mapDispatchToConnectorProps(dispatch) {
      return bindActionCreators({
        connectReadModel: connectReadModelAction,
        disconnectReadModel: disconnectReadModelAction
      }, dispatch);
    };

    var ReadModelConnector = connect(mapStateToConnectorProps, mapDispatchToConnectorProps)(ReadModelContainer);
    ReadModelConnector.mapStateToOptions = mapStateToOptions;
    hoistNonReactStatic(ReadModelConnector, ReadModelContainer);
    return connectResolveAdvanced(ReadModelConnector);
  };
};

export default connectReadModel;
//# sourceMappingURL=connect-read-model.js.map