import setEntry from 'lodash.set';
import unsetEntry from 'lodash.unset';
import getByPath from 'lodash.get';
import cloneDeep from 'lodash.clonedeep';
import getHash from '../internal/get-hash';
import { QUERY_READMODEL_REQUEST, QUERY_READMODEL_SUCCESS, QUERY_READMODEL_FAILURE, DROP_READMODEL_STATE, INIT_READMODEL } from '../internal/action-types';
import { ResultStatus } from '../types';
export var namedSelectors = 'namedSelectors';
export var badSelectorDrain = 'badSelectorDrain';
export var builtInSelectors = 'builtInSelectors';

var getSelector = function getSelector(action) {
  return action.selectorId || action;
};

var getEntryPath = function getEntryPath(selector) {
  if (typeof selector === 'string') {
    return namedSelectors + "." + getHash(selector);
  }

  var _selector$query = selector.query,
      name = _selector$query.name,
      resolver = _selector$query.resolver,
      args = _selector$query.args;
  return builtInSelectors + "." + getHash(name) + "." + getHash(resolver) + "." + getHash(args);
};

export var getEntry = function getEntry(state, selector, placeholder) {
  return getByPath(state, getEntryPath(selector), placeholder);
};
var initialState = {};
export var reducer = function reducer(state, action) {
  var _action$error$message, _action$error;

  if (state === void 0) {
    state = initialState;
  }

  switch (action.type) {
    case QUERY_READMODEL_REQUEST:
      return setEntry(cloneDeep(state), getEntryPath(getSelector(action)), {
        status: ResultStatus.Requested,
        data: action.initialState
      });

    case QUERY_READMODEL_SUCCESS:
      return setEntry(cloneDeep(state), getEntryPath(getSelector(action)), {
        status: ResultStatus.Ready,
        data: action.result.data
      });

    case QUERY_READMODEL_FAILURE:
      return setEntry(cloneDeep(state), getEntryPath(getSelector(action)), {
        status: ResultStatus.Failed,
        data: null,
        error: (_action$error$message = (_action$error = action.error) == null ? void 0 : _action$error.message) != null ? _action$error$message : 'unknown error'
      });

    case INIT_READMODEL:
      return setEntry(cloneDeep(state), getEntryPath(getSelector(action)), {
        status: ResultStatus.Initial,
        data: action.initialState
      });

    case DROP_READMODEL_STATE:
      var newState = cloneDeep(state);
      unsetEntry(newState, getEntryPath(getSelector(action)));
      return newState;

    default:
      return state;
  }
};
//# sourceMappingURL=read-model-reducer.js.map