"use strict";

exports.__esModule = true;
exports.useReduxCommand = useReduxCommand;

var _react = require("react");

var _reactRedux = require("react-redux");

var _core = require("@resolve-js/core");

var _reactHooks = require("@resolve-js/react-hooks");

var _helpers = require("../helpers");

var _actions = require("./actions");

var internalActions = {
  request: function request(command) {
    return (0, _actions.sendCommandRequest)(command, true);
  },
  success: _actions.sendCommandSuccess,
  failure: _actions.sendCommandFailure
};
var defaultCommandOptions = {};
var defaultHookOptions = {};

function isBuilder(query) {
  return typeof query === 'function';
}

function useReduxCommand(command, options, dependencies) {
  var actualOptions = (0, _helpers.isOptions)(options) ? options : defaultHookOptions;
  var actualActionCreators = actualOptions.actions || internalActions;
  var actualDependencies = (0, _core.firstOfType)(_helpers.isDependencies, options, dependencies);
  var request = actualActionCreators.request,
      success = actualActionCreators.success,
      failure = actualActionCreators.failure;

  var callback = function callback(error, result, executedCommand) {
    if (error || result == null) {
      if (typeof failure === 'function') {
        if (error) {
          dispatch(failure(executedCommand, error));
        } else {
          dispatch(failure(executedCommand, new Error("null response")));
        }
      }
    } else {
      if (typeof success === 'function') {
        dispatch(success(executedCommand, result));
      }
    }
  };

  var dispatch = (0, _reactRedux.useDispatch)();
  var executor = actualDependencies ? (0, _reactHooks.useCommandBuilder)(function (command) {
    return command;
  }, actualOptions.commandOptions || defaultCommandOptions, callback, actualDependencies) : (0, _reactHooks.useCommandBuilder)(function (command) {
    return command;
  }, actualOptions.commandOptions || defaultCommandOptions, callback);
  return (0, _react.useMemo)(function () {
    return {
      execute: function execute() {
        var dispatchRequest = function dispatchRequest(command) {
          if (typeof request === 'function') {
            dispatch(request(command));
          }
        };

        var plainCommand = isBuilder(command) ? command.apply(void 0, arguments) : command;
        dispatchRequest(plainCommand);
        executor(plainCommand);
      }
    };
  }, [executor, dispatch]);
}
//# sourceMappingURL=use-redux-command.js.map