"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.reducer = exports.getEntry = exports.builtInSelectors = exports.badSelectorDrain = exports.namedSelectors = void 0;

var _lodash = _interopRequireDefault(require("lodash.set"));

var _lodash2 = _interopRequireDefault(require("lodash.unset"));

var _lodash3 = _interopRequireDefault(require("lodash.get"));

var _lodash4 = _interopRequireDefault(require("lodash.clonedeep"));

var _getHash = _interopRequireDefault(require("../internal/get-hash"));

var _actionTypes = require("../internal/action-types");

var _types = require("../types");

var namedSelectors = 'namedSelectors';
exports.namedSelectors = namedSelectors;
var badSelectorDrain = 'badSelectorDrain';
exports.badSelectorDrain = badSelectorDrain;
var builtInSelectors = 'builtInSelectors';
exports.builtInSelectors = builtInSelectors;

var getSelector = function getSelector(action) {
  return action.selectorId || action;
};

var getEntryPath = function getEntryPath(selector) {
  if (typeof selector === 'string') {
    return namedSelectors + "." + (0, _getHash["default"])(selector);
  }

  var _selector$query = selector.query,
      name = _selector$query.name,
      resolver = _selector$query.resolver,
      args = _selector$query.args;
  return builtInSelectors + "." + (0, _getHash["default"])(name) + "." + (0, _getHash["default"])(resolver) + "." + (0, _getHash["default"])(args);
};

var getEntry = function getEntry(state, selector, placeholder) {
  return (0, _lodash3["default"])(state, getEntryPath(selector), placeholder);
};

exports.getEntry = getEntry;
var initialState = {};

var reducer = function reducer(state, action) {
  var _action$error$message, _action$error;

  if (state === void 0) {
    state = initialState;
  }

  switch (action.type) {
    case _actionTypes.QUERY_READMODEL_REQUEST:
      return (0, _lodash["default"])((0, _lodash4["default"])(state), getEntryPath(getSelector(action)), {
        status: _types.ResultStatus.Requested,
        data: action.initialState
      });

    case _actionTypes.QUERY_READMODEL_SUCCESS:
      return (0, _lodash["default"])((0, _lodash4["default"])(state), getEntryPath(getSelector(action)), {
        status: _types.ResultStatus.Ready,
        data: action.result.data
      });

    case _actionTypes.QUERY_READMODEL_FAILURE:
      return (0, _lodash["default"])((0, _lodash4["default"])(state), getEntryPath(getSelector(action)), {
        status: _types.ResultStatus.Failed,
        data: null,
        error: (_action$error$message = (_action$error = action.error) == null ? void 0 : _action$error.message) != null ? _action$error$message : 'unknown error'
      });

    case _actionTypes.INIT_READMODEL:
      return (0, _lodash["default"])((0, _lodash4["default"])(state), getEntryPath(getSelector(action)), {
        status: _types.ResultStatus.Initial,
        data: action.initialState
      });

    case _actionTypes.DROP_READMODEL_STATE:
      var newState = (0, _lodash4["default"])(state);
      (0, _lodash2["default"])(newState, getEntryPath(getSelector(action)));
      return newState;

    default:
      return state;
  }
};

exports.reducer = reducer;
//# sourceMappingURL=read-model-reducer.js.map